@@ -1,7 +1,8 @@
-using Mono.Data.SqliteClient;
+using System.Threading;
 using System.Collections;
 using System.IO;
 using System;
+using Banshee.Database;
 
 
 // All kinds of database items subclass from this.
@@ -124,22 +125,21 @@ public abstract class DbStore {
 		}
 	}
 
-	// Sqlite stuff.
 
-	SqliteConnection connection;
-	protected SqliteConnection Connection {
+	QueuedSqliteDatabase database;
+	protected QueuedSqliteDatabase Database {
 		get {
-			return connection;
+			return database;
 		}
 	}
 
 
 	// Constructor.
 
-	public DbStore (SqliteConnection connection,
+	public DbStore (QueuedSqliteDatabase database,
 			bool cache_is_immortal)
 	{
-		this.connection = connection;
+		this.database = database;
 		this.cache_is_immortal = cache_is_immortal;
 
 		item_cache = new Hashtable ();
@@ -153,39 +153,6 @@ public abstract class DbStore {
 	// If you have made changes to "obj", you have to invoke Commit() to have the changes
 	// saved into the database.
 	public abstract void Commit (DbItem item);
-
-
-	// Utility methods.
-
-	// Subclasses use this to generate valid SQL commands.
-	protected static string SqlString (string s) {
-		return s.Replace ("'", "''");
-	}
-
-	public void BeginTransaction () {
-		SqliteCommand command = new SqliteCommand ();
-		command.Connection = Connection;
-		command.CommandText = "BEGIN TRANSACTION";
-		command.ExecuteScalar ();
-		command.Dispose ();
-	}
-	
-	public void CommitTransaction () {
-		SqliteCommand command = new SqliteCommand ();
-		command.Connection = Connection;
-		command.CommandText = "COMMIT TRANSACTION";
-		command.ExecuteScalar ();
-		command.Dispose ();
-	}
-	
-	public void RollbackTransaction () {
-		SqliteCommand command = new SqliteCommand ();
-		command.Connection = Connection;
-		command.CommandText = "ROLLBACK";
-		command.ExecuteScalar ();
-		command.Dispose ();
-	}
-
 }
 
 
@@ -195,16 +162,27 @@ public class Db : IDisposable {
 
 	TagStore tag_store;
 	PhotoStore photo_store;
- 	ImportStore import_store;
+ 	RollStore roll_store;
+	ExportStore export_store;
+ 	JobStore job_store;
  	MetaStore meta_store;
 	bool empty;
+	string path;
 
 	public TagStore Tags {
 		get { return tag_store; }
 	}
 
-	public ImportStore Imports {
-		get { return import_store; }
+	public RollStore Rolls {
+		get { return roll_store; }
+	}
+
+	public ExportStore Exports {
+		get { return export_store; }
+	}
+
+	public JobStore Jobs {
+		get { return job_store; }
 	}
 
 	public PhotoStore Photos {
@@ -219,100 +197,62 @@ public class Db : IDisposable {
 	// therefore how likely corruption is in the event of power loss.
 	public bool Sync {
 		set {
-			SqliteCommand command = new SqliteCommand ();
-			command.Connection = sqlite_connection;
-			command.CommandText = "PRAGMA synchronous = " + (value ? "ON" : "OFF");
-			command.ExecuteScalar ();
-			command.Dispose ();
+			string query = "PRAGMA synchronous = " + (value ? "ON" : "OFF");
+			Database.ExecuteNonQuery(query);
 		}
 	}
 
-	public void BeginTransaction () {
-		SqliteCommand command = new SqliteCommand ();
-		command.Connection = sqlite_connection;
-		command.CommandText = "BEGIN TRANSACTION";
-		command.ExecuteScalar ();
-		command.Dispose ();
-	}
-	
-	public void CommitTransaction () {
-		SqliteCommand command = new SqliteCommand ();
-		command.Connection = sqlite_connection;
-		command.CommandText = "COMMIT TRANSACTION";
-		command.ExecuteScalar ();
-		command.Dispose ();
-	}
-	
-	public void RollbackTransaction () {
-		SqliteCommand command = new SqliteCommand ();
-		command.Connection = sqlite_connection;
-		command.CommandText = "ROLLBACK";
-		command.ExecuteScalar ();
-		command.Dispose ();
+	QueuedSqliteDatabase database;
+	public QueuedSqliteDatabase Database {
+		get { return database; }
 	}
 
-	SqliteConnection sqlite_connection;
-	public SqliteConnection Connection {
-		get { return sqlite_connection; }
-	}
 
-	private static int GetFileVersion (string path)
+	public string Repair ()
 	{
-		using (Stream stream = File.OpenRead (path)) {
-			byte [] data = new byte [15];
-			stream.Read (data, 0, data.Length);
+		string backup_path = path;
+		int i = 0;
 
-			string magic = System.Text.Encoding.ASCII.GetString (data, 0, data.Length);
-
-			switch (magic) {
-			case "SQLite format 3":
-				return 3;
-			case "** This file co":
-				return 2;
-			default:
-				return -1;
-			}
+		while (File.Exists (backup_path)) {
+			backup_path = String.Format ("{0}-{1}-{2}{3}",
+						     Path.GetFileNameWithoutExtension (path),
+						     System.DateTime.Now.ToString ("yyyyMMdd"),
+						     i++,
+						     Path.GetExtension (path));
 		}
+		
+		File.Move (path, backup_path);
+		Init (path, true);
+
+		return backup_path;
 	}
 
 	public void Init (string path, bool create_if_missing)
 	{
 		bool new_db = ! File.Exists (path);
-		string version_string = "";
+		this.path = path;
 
 		if (new_db && ! create_if_missing)
 			throw new Exception (path + ": File not found");
 
-		if (! new_db) {
-			int version = Db.GetFileVersion (path);
-			// FIXME: we should probably display and error dialog if the version
-			// is anything other than the one we were built with, but for now at least
-			// use the right version.
+		database = new QueuedSqliteDatabase(path);	
 
-			if (version < 2)
-				throw new Exception ("Unsupported database version");
-			
-			version_string = String.Format (",version={0}", version);
-
-			if (version == 2)
-				version_string += ",encoding=UTF-8";
-		}
-		
-		sqlite_connection = new SqliteConnection ();
-		sqlite_connection.ConnectionString = "URI=file:" + path + version_string;
-
-		sqlite_connection.Open ();
-		
 		// Load or create the meta table
- 		meta_store = new MetaStore (sqlite_connection, new_db);
+ 		meta_store = new MetaStore (Database, new_db);
 
 		// Update the database schema if necessary
 		FSpot.Database.Updater.Run (this);
 
-		tag_store = new TagStore (sqlite_connection, new_db);
-		import_store = new ImportStore (sqlite_connection, new_db);
- 		photo_store = new PhotoStore (sqlite_connection, new_db, tag_store);
+		Database.BeginTransaction ();
+
+		tag_store = new TagStore (Database, new_db);
+		roll_store = new RollStore (Database, new_db);
+		export_store = new ExportStore (Database, new_db);
+		job_store = new JobStore (Database, new_db);
+ 		photo_store = new PhotoStore (Database, new_db);
 		
+		Database.CommitTransaction ();
+
 		empty = new_db;
 	}
 
@@ -322,19 +262,51 @@ public class Db : IDisposable {
 		}
 	}
 
-	public void Dispose () {}
+	public void Dispose ()
+	{
+		Database.Dispose ();
+	}
+
+	public void BeginTransaction()
+	{
+		Database.BeginTransaction ();
+	}
+
+	public void CommitTransaction()
+	{
+		Database.CommitTransaction ();
+	}
+
+	public void RollbackTransaction()
+	{
+		Database.RollbackTransaction ();
+	}
+
 }
 
 
 public class DbUtils {
+#if USE_CORRECT_FUNCTION
+	public static DateTime DateTimeFromUnixTime (long unix_time)
+	{
+		DateTime date_time = new DateTime (1970, 1, 1);
+		return date_time.AddSeconds (unix_time).ToLocalTime ();
+	}
+
+	public static long UnixTimeFromDateTime (DateTime date_time)
+	{
+		return (long) (date_time.ToUniversalTime () - new DateTime (1970, 1, 1)).TotalSeconds;
+	}
+#else
 	public static DateTime DateTimeFromUnixTime (long unix_time)
 	{
 		DateTime date_time = new DateTime (1970, 1, 1).ToLocalTime ();
 		return date_time.AddSeconds (unix_time);
 	}
-
+	
 	public static long UnixTimeFromDateTime (DateTime date_time)
 	{
 		return (long) (date_time - new DateTime (1970, 1, 1).ToLocalTime ()).TotalSeconds;
 	}
+#endif
 }
