@@ -341,7 +341,7 @@ namespace FSpot {
 			password = password_entry.Text;
 			username = username_entry.Text;
 
-			if (name == "" || url == "" || password == "" || username == "")
+			if (name == String.Empty || url == String.Empty || password == String.Empty || username == String.Empty)
 				add_button.Sensitive = false;
 			else
 				add_button.Sensitive = true;
@@ -509,13 +509,13 @@ namespace FSpot {
 		{
 			if (gallery.Version == GalleryVersion.Version1) {
 				if (gallery.Albums.Count == 0 || album_optionmenu.History <= 0) {
-					parent = "";
+					parent = String.Empty;
 				} else {
 					parent = ((Album) gallery.Albums [album_optionmenu.History-1]).Name;
 				}
 			} else {
 				if (gallery.Albums.Count == 0 || album_optionmenu.History < 0) {
-					parent = "";
+					parent = String.Empty;
 				} else {
 					parent = ((Album) gallery.Albums [album_optionmenu.History]).Name;
 				}
@@ -524,7 +524,7 @@ namespace FSpot {
 			description = description_entry.Text;
 			title = title_entry.Text;
 
-			if (name == "" || title == "")
+			if (name == String.Empty || title == String.Empty)
 				add_button.Sensitive = false;
 			else
 				add_button.Sensitive = true;
@@ -542,8 +542,12 @@ namespace FSpot {
 	}
 
 	
-	public class GalleryExport : GladeDialog {
-		public GalleryExport (IBrowsableCollection selection) : base ("gallery_export_dialog")
+	public class GalleryExport : GladeDialog, FSpot.Extensions.IExporter {
+		public GalleryExport () : base ("gallery_export_dialog")
+		{
+		}
+
+		public void Run (IBrowsableCollection selection)
 		{
 			this.items = selection.Items;
 			album_button.Sensitive = false;
@@ -680,6 +684,8 @@ namespace FSpot {
 				System.Console.WriteLine ("Starting upload");
 				
 				FilterSet filters = new FilterSet ();
+				if (account.Version == GalleryVersion.Version1)
+					filters.Add (new WhiteListFilter (new string []{".jpg", ".jpeg", ".png", ".gif"}));
 				if (scale)
 					filters.Add (new ResizeFilter ((uint) size));
 				else if (rotate)
