@@ -215,7 +215,11 @@ namespace FSpot {
 				update_request_attributes["name"] = keyring_item_name;
 				update_request_attributes["username"] = account.Username;
 
-				Ring.CreateItem(keyring, ItemType.GenericSecret, keyring_item_name, update_request_attributes, account.Password, true);
+				try {
+					Ring.CreateItem(keyring, ItemType.GenericSecret, keyring_item_name, update_request_attributes, account.Password, true);
+				} catch {
+					continue;
+				}
 			}
 		}
 
@@ -402,10 +406,7 @@ namespace FSpot {
 				public_album = public_check.Active;
 				
 				try {
-					if (public_album)
-						picasa.CreateAlbum (title, description, AlbumAccess.Public);
-					else	
-						picasa.CreateAlbum (title, description, AlbumAccess.Private);
+					picasa.CreateAlbum (System.Web.HttpUtility.HtmlEncode (title), description, public_album ? AlbumAccess.Public : AlbumAccess.Private);
 				} catch (System.Exception e) {
 					HigMessageDialog md =
 					new HigMessageDialog (Dialog, 
@@ -465,6 +466,7 @@ namespace FSpot {
 			LoadPreference (Preferences.EXPORT_PICASAWEB_ROTATE);
 			LoadPreference (Preferences.EXPORT_PICASAWEB_BROWSER);
 //			LoadPreference (Preferences.EXPORT_PICASAWEB_META);
+			LoadPreference (Preferences.EXPORT_PICASAWEB_TAG);
 		}
 		
 		private bool scale;
@@ -472,6 +474,7 @@ namespace FSpot {
 		private bool browser;
 		private bool rotate;
 //		private bool meta;
+		private bool export_tag;
 		private bool connect = false;
 
 		private long approx_size = 0;
@@ -500,6 +503,7 @@ namespace FSpot {
 		[Glade.Widget] Gtk.CheckButton scale_check;
 		[Glade.Widget] Gtk.CheckButton rotate_check;
 //		[Glade.Widget] Gtk.CheckButton meta_check;
+		[Glade.Widget] Gtk.CheckButton tag_check;
 		
 		[Glade.Widget] Gtk.SpinButton size_spin;
 
@@ -531,6 +535,7 @@ namespace FSpot {
 			browser = browser_check.Active;
 			rotate = rotate_check.Active;
 //			meta = meta_check.Active;
+			export_tag = tag_check.Active;
 
 			if (account != null) { 
 				//System.Console.WriteLine ("history = {0}", album_optionmenu.History);
@@ -551,6 +556,7 @@ namespace FSpot {
 				Preferences.Set (Preferences.EXPORT_PICASAWEB_ROTATE, rotate);
 				Preferences.Set (Preferences.EXPORT_PICASAWEB_BROWSER, browser);
 //				Preferences.Set (Preferences.EXPORT_GALLERY_META, meta);
+				Preferences.Set (Preferences.EXPORT_PICASAWEB_TAG, export_tag);
 			}
 		}
 		
@@ -562,9 +568,9 @@ namespace FSpot {
 		private void HandleUploadProgress(object o, UploadProgressEventArgs args)
 		{
 				if (approx_size == 0)
-					progress_dialog.ProgressText = System.String.Format ("{0} Sent",SizeUtil.ToHumanReadable(args.BytesSent));
+					progress_dialog.ProgressText = System.String.Format (Catalog.GetString ("{0} Sent"), SizeUtil.ToHumanReadable(args.BytesSent));
 				else
-					progress_dialog.ProgressText = System.String.Format ("{0} of approx. {1}", SizeUtil.ToHumanReadable(sent_bytes + args.BytesSent), SizeUtil.ToHumanReadable(approx_size));
+					progress_dialog.ProgressText = System.String.Format (Catalog.GetString ("{0} of approx. {1}"), SizeUtil.ToHumanReadable(sent_bytes + args.BytesSent), SizeUtil.ToHumanReadable(approx_size));
 				progress_dialog.Fraction = ((photo_index - 1) / (double) items.Length) + (args.BytesSent / (args.BytesTotal * (double) items.Length));
 		}
 
@@ -585,6 +591,8 @@ namespace FSpot {
 			if (rotate)
 				filters.Add (new OrientationFilter ());
 
+			Array.Sort (items as Photo[], new Photo.CompareDateName ());
+
 			while (photo_index < items.Length) {
 				try {
 					IBrowsableItem item = items[photo_index];
@@ -618,7 +626,7 @@ namespace FSpot {
 
 					request.Dispose ();
 					//tagging
-					if (item.Tags != null)
+					if (item.Tags != null && export_tag)
 						foreach (Tag tag in item.Tags)
 							picture.AddTag (tag.Name);
 
@@ -868,6 +876,11 @@ namespace FSpot {
 //				if (meta_check.Active != (bool) val)
 //					meta_check.Active = (bool) val;
 //				break;
+			
+			case Preferences.EXPORT_PICASAWEB_TAG:
+				if (tag_check.Active != (bool) val)
+					tag_check.Active = (bool) val;
+				break;
 			}
 		}
 	}
